/* Copyright 2005 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: App.cpp,v 1.1 2005/04/17 10:49:27 tonic Exp $
 * $Revision: 1.1 $
 */

#include <time.h>
#include "main.h"


#define BLURMIX(a,b,n) ((a) * ((n) - 1) + (b)) / (n)


App * App::mSingleton = NULL;


App::App() :
    mState(0),
    mScreen(NULL),
    mAlive(TRUE),
    mTime(0),
    mStartTime(0),
    mNextUpdateTime(0),
    mCurrentStateHandler(NULL),
    mCustomEventFilter(NULL)
{
}


App::~App()
{
}


void App::setInstance(App *singleton)
{
    assert(mSingleton == NULL);
    mSingleton = singleton;
}


static int sEventFilter(const SDL_Event * /*event*/)
{
    return 1;
}


static const char *arrow[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "                                ",
  "                                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "               X                ",
  "  .......     X.X     .......   ",
  "               X                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "               .                ",
  "                                ",
  "                                ",
  "                                ",
  "15,15"
};

// taken from some SDL example code
static SDL_Cursor * sConstructMouseCursor(const char *image[])
{
    int i, row, col;
    Uint8 data[4*32];
    Uint8 mask[4*32];
    int hot_x, hot_y;

    i = -1;
    for ( row=0; row<32; ++row ) {
        for ( col=0; col<32; ++col ) {
            if ( col % 8 ) {
                data[i] <<= 1;
                mask[i] <<= 1;
            } else {
                ++i;
                data[i] = mask[i] = 0;
            }
            switch (image[4+row][col]) {
            case 'X':
                data[i] |= 0x01;
                mask[i] |= 0x01;
                //          k[i] |= 0x01;
                break;
            case '.':
                mask[i] |= 0x01;
                break;
            case '*':
                data[i] |= 0x01;
            case ' ':
                break;
            }
        }
    }
    sscanf(image[4+row], "%d,%d", &hot_x, &hot_y);
    return SDL_CreateCursor(data, mask, 32, 32, hot_x, hot_y);
}


static void sCreateMouseCursor()
{
    SDL_SetCursor(sConstructMouseCursor(arrow));
}


BOOL App::init()
{
    setInstance(this);

    SDL_WM_SetCaption(getCaption(), "");
    sCreateMouseCursor();

    mScreen = SDL_GetVideoSurface();
    assert(mScreen != NULL);

    SDL_SetEventFilter(sEventFilter);

    resetTime();
    srand(time(NULL));

    return TRUE;
}


void App::run()
{
    while (isAlive())
    {
        Uint32 frameUpdateStart = SDL_GetTicks();
        SDL_Event event;
        while (SDL_PollEvent(&event))
        {
            SDLKey keysym;
            switch (event.type)
            {
            case SDL_KEYDOWN:
                keysym = event.key.keysym.sym;
                if (keysym == SDLK_ESCAPE)
                    setAlive(FALSE);
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onKeyDown(keysym);
                break;

            case SDL_KEYUP:
                keysym = event.key.keysym.sym;
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onKeyUp(keysym);
                break;

            case SDL_MOUSEMOTION:
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onMouseMotion(event.motion);
                break;

            case SDL_MOUSEBUTTONUP:
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onMouseUp(event.button);
                break;
                
            case SDL_MOUSEBUTTONDOWN:
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onMouseDown(event.button);
                break;

            case SDL_QUIT:
                setAlive(FALSE);
                break;
            }
        }

        BOOL updated = FALSE;
        UINT32 time = SDL_GetTicks() - mStartTime;
        mTime = BLURMIX(mTime, time, 4);

        updated |= update();

        if (updated)
        {
            if (mCurrentStateHandler != NULL)
                mCurrentStateHandler->render(mScreen);
            //SDL_Flip(mScreen);    // SDL_UpdateRect is called from GameState
            glFlush();
            SDL_GL_SwapBuffers();
        }
        //else
        //    SDL_Delay(1);

        Uint32 frameUpdateEnd = SDL_GetTicks();
        if (frameUpdateStart == frameUpdateEnd)
            SDL_Delay(1);

        GLenum error = glGetError();
        if (error != GL_NO_ERROR)
            errorMessage("GL Error", "GL Error");
    }
}


BOOL App::update()
{
    if (mTime < mNextUpdateTime)
        return FALSE;

    if (mCurrentStateHandler == NULL)
    {
        mNextUpdateTime = mTime;
        return FALSE;
    }

    // TODO: Update this so that FALSE is returned if the current
    //       state handler returned FALSE for each iteration?
    while (mNextUpdateTime < mTime)
    {
        mNextUpdateTime += 1000 / mCurrentStateHandler->getUpdateFPS();
        
        mCurrentStateHandler->update(mNextUpdateTime);
    }

    return TRUE;
}


void App::resetTime()
{
    mNextUpdateTime = 0;
    mStartTime = SDL_GetTicks();
    mTime = 0;
}


void App::setState(STATE_TYPE state)
{
    if (mCurrentStateHandler != NULL)
        mCurrentStateHandler->onDeactivate();

    mCurrentStateHandler = getStateHandler(state);

    mState = state;
    if (mCurrentStateHandler != NULL)
        mCurrentStateHandler->onActivate(mTime);
}
