/* Copyright 2005 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: GameApp.h,v 1.3 2005/04/17 22:45:02 tonic Exp $
 * $Revision: 1.3 $
 */

#ifndef GAMEAPP_H_INCL
#define GAMEAPP_H_INCL


#include "App.h"


#define CAPTION_STR "Light Warrior " VERSION_STR

#define DATA_PREFIX "data/"


class GameApp : public App
{
public:

    enum STATE_ENUM
    {
        // App initializes state 0 first.
        STATE_INITIAL = 0,
        STATE_TITLE,
        STATE_GAME_RUN
    };

    GameApp();
    virtual ~GameApp();

    virtual BOOL init();
    virtual void deinit();

    // GameApp:

    static GameApp * getInstance() { return (GameApp *)App::getInstance(); }

    UIGraphics * getGraphics() { return mGraphics; }

    enum FONT_ENUM
    {
        FONT_SMALL = 0,
        FONT_MEDIUM,
        FONT_BIG,
        FONT_COUNT
    };

    UIFont * getSmallFont() { return mFonts[FONT_SMALL]; }
    UIFont * getMediumFont() { return mFonts[FONT_MEDIUM]; }
    UIFont * getBigFont() { return mFonts[FONT_BIG]; }

    void setBestScore(INT32 score);
    INT32 getBestScore() { return mBestScore; }

protected:

    // From App:
    virtual char * getCaption();
    virtual State * getStateHandler(STATE_TYPE state);

    // GameApp:

    typedef std::map<STATE_TYPE, State *> StateMap;

    void initAndAddState(STATE_TYPE stateId, State *newState);
    void deinitAndDeleteStates();

    void deleteFonts();

    StateMap mStates;
    //! FALSE if initialization and registration of any of the states has failed.
    BOOL mStateInitStatus;

    //UIGraphicsOpenGLSDLCFL *mGraphics;
    UIGraphicsOpenGLSDL *mGraphics;
    UIFont *mFonts[FONT_COUNT];

    INT32 mBestScore;
};


#endif // !GAMEAPP_H_INCL
