/* Copyright 2005 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: GameState.h,v 1.2 2005/04/17 16:14:09 tonic Exp $
 * $Revision: 1.2 $
 */

#ifndef GAMESTATE_H_INCL
#define GAMESTATE_H_INCL


class GameState : public State, public UIEventListener
{
public:

    GameState(UIGraphics *graphics);
    virtual ~GameState();

    // From State:

    virtual BOOL init();
    virtual void deinit();

    virtual BOOL update(UINT32 time);
    virtual void render(SDL_Surface *screen);

    virtual void onActivate(UINT32 time);

    virtual void onKeyDown(const SDLKey &key);
    virtual void onMouseDown(const SDL_MouseButtonEvent &event);
    virtual void onMouseUp(const SDL_MouseButtonEvent &event);
    virtual void onMouseMotion(const SDL_MouseMotionEvent &event);

    // From UIEventListener:

    virtual void onAction(UIComponent *component);

    // Local defs:

    BOOL loadMap(char *mapFilename, char *tileImageFilename);

    void startLevel(FLOAT scrollSpeed);

    INT32 getLevelLightCount() { return mLevelLightCount; }
    INT32 getLightsKilled() { return mLightsKilled; }
    INT32 getBulletsShot() { return mBulletsShot; }

protected:

    enum STATE
    {
        STATE_RUNNING = 0,
        STATE_FINISHED,
    };

    void destroyMap();

    void reload();

    void renderLevel();
    void renderBulletSystem();

    struct Tile
    {
        UINT16 base;
        // Light index, 0 == no light, other values indicate light tile.
        // It is hardcoded that if lowest bit is 0, light is active,
        // and if lowest bit is 1, the light is broken.
        UINT16 lights;
    };

    class Light;

    UINT32 mTime, mStartTime;
    UI mUI;
    Levels mLevels;
    ParticleSystem mParticleSystem;

    Light *mLights;

    FLOAT mScrollPos, mScrollSpeed;

    UINT32 mMapWidth, mMapHeight;
    Tile *mTileMap;
    UIImage *mTileImage;

    UIImage *mBulletImage;
    UIImage *mButtonUpImage, *mButtonDownImage;
    UIImage *mSmallButtonUpImage, *mSmallButtonDownImage;
    UIButton *mContinueButton;

    UIGraphics *mGraphics;

    INT32 mState;

    INT32 mBullets;
    INT32 mLevelLightCount;
    INT32 mLightsKilled;
    INT32 mBulletsShot;
    INT32 mReloadTime;

    BOOL mNotifyContinueFlag;
};


#endif // !GAMESTATE_H_INCL
