/* Copyright 2005 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: TitleState.cpp,v 1.3 2005/04/17 22:45:02 tonic Exp $
 * $Revision: 1.3 $
 */

#include "main.h"


TitleState::TitleState() :
    mPlayButton(NULL),
    mEditorButton(NULL),
    mExitButton(NULL),
    mButtonUpImage(NULL),
    mButtonDownImage(NULL)
{
}


TitleState::~TitleState()
{
    deinit();
}


BOOL TitleState::init()
{
    BOOL success = TRUE;

    GameApp *ga = GameApp::getInstance();
    INT scrW = ga->getScreenWidth(), scrH = ga->getScreenHeight();

    UIGraphics *graphics = ga->getGraphics();
    mUI.setGraphics(graphics);

    UIFont *medium = ga->getMediumFont();
    UIFont *small = ga->getSmallFont();
    UIFont *big = ga->getBigFont();

    mButtonUpImage = graphics->loadImage(DATA_PREFIX "button_up.png");
    mButtonDownImage = graphics->loadImage(DATA_PREFIX "button_down.png");
    if (!mButtonDownImage || !mButtonDownImage)
        success = FALSE;

    const UINT buttonWidth = mButtonUpImage->getWidth();
    const UINT buttonHeight = mButtonUpImage->getHeight();
    const UINT gap = 50;

    mPlayButton = new UIButton(scrW / 2 - buttonWidth / 2, scrH / 2 + buttonHeight,
                               mButtonUpImage, mButtonDownImage, NULL,
                               medium, "Play");
    mPlayButton->setColor(0.9f, 1, 0.9f, 1);
    mPlayButton->addEventListener(this);
    mUI.addUIComponent(mPlayButton);
    if (!mPlayButton)
        success = FALSE;

    mExitButton = new UIButton(scrW / 2 - buttonWidth / 2, scrH / 2 + buttonHeight * 5 / 2,
                               mButtonUpImage, mButtonDownImage, NULL,
                               medium, "Exit");
    mExitButton->setColor(1, 0.9f, 0.9f, 1);
    mExitButton->addEventListener(this);
    mUI.addUIComponent(mExitButton);
    if (!mExitButton)
        success = FALSE;

    const UINT lgap = 8 + big->getFontHeight();
    UILabel *label;

    label = new UILabel(scrW / 2, gap * 2, NULL, big,
                        "Light Warrior", UIFont::ALIGN_HORIZ_CENTER);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    label = new UILabel(scrW / 2, gap * 2 + lgap, NULL, small,
                        "Version " VERSION_STR, UIFont::ALIGN_HORIZ_CENTER);
    label->setItemColor(0.7f, 0.7f, 0.7f, 1);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    label = new UILabel(scrW / 2, gap * 2 + lgap + small->getFontHeight(), NULL, medium,
                        "Shoot all the lights!", UIFont::ALIGN_HORIZ_CENTER);
    label->setItemColor(1, 1, 0.75f, 1);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    mScoreLabel = new UILabel(scrW / 2, gap * 2 + lgap * 2, NULL, small,
                              "", UIFont::ALIGN_HORIZ_CENTER);
    mScoreLabel->setItemColor(0.75f, 1, 0.75f, 1);
    mUI.addUIComponent(mScoreLabel);
    if (!label)
        success = FALSE;

    label = new UILabel(scrW / 2, scrH - 1, NULL, small,
                        "Copyright 2005 Jetro Lauha - http://jet.ro",
                        UIFont::ALIGN_HORIZ_CENTER | UIFont::ALIGN_BOTTOM);
    label->setItemColor(0.75f, 0.75f, 0.75f, 1);
    mUI.addUIComponent(label);
    if (!label)
        success = FALSE;

    mUI.activateAllComponents();

    return success;
}


void TitleState::onActivate(UINT32 /*time*/)
{
    setMusic(TRUE, MUSIC_MENU);

    GameApp *ga = GameApp::getInstance();
    INT32 bestScore = ga->getBestScore();
    if (bestScore > 0)
    {
        char tmp[512];
        sprintf(tmp, "Best score: %d", bestScore);
        mScoreLabel->setText(tmp);
    }
}


void TitleState::deinit()
{
    mUI.deleteAllUIComponents();

    delete mButtonUpImage;
    mButtonUpImage = NULL;
    delete mButtonDownImage;
    mButtonDownImage = NULL;
}


BOOL TitleState::update(UINT32 time)
{
    mTime = time;

    return TRUE;
}


void TitleState::render(SDL_Surface *screen)
{
    glClearColor(0, 0, 0, 1);
    glClear(GL_COLOR_BUFFER_BIT);

    GameApp *app = (GameApp *)App::getInstance();

    mUI.render();
}


void TitleState::onKeyDown(const SDLKey &key)
{
    if (key == SDLK_SPACE || key == SDLK_KP_ENTER || key == SDLK_RETURN)
        gameStart();
}


void TitleState::onMouseDown(const SDL_MouseButtonEvent &event)
{
    mUI.handleUIPointerDown(event.x, event.y);
}


void TitleState::onMouseUp(const SDL_MouseButtonEvent &event)
{
    mUI.handleUIPointerUp(event.x, event.y);
}


void TitleState::onMouseMotion(const SDL_MouseMotionEvent &event)
{
    mUI.handleUIPointerMove(event.x, event.y);
}


void TitleState::gameStart()
{
    GameApp *app = (GameApp *)App::getInstance();
    if (app->isAlive())
        app->setState(GameApp::STATE_GAME_RUN);
}


void TitleState::onAction(UIComponent *component)
{
    if (component == mPlayButton)
        gameStart();
    else if (component == mExitButton)
        App::getInstance()->setAlive(FALSE);
}
