/* Copyright 2005 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: main.cpp,v 1.7 2005/04/17 22:50:54 tonic Exp $
 * $Revision: 1.7 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

#include "main.h"
#include "GameApp.h"
#include "SDL_image.h"


static SDL_Surface *sScreen;


static SDL_Surface * setVideoMode(int width, int height, int bpp, int fullscreen, int fsaa)
{
    Uint32 videoFlags = 0;
    int rSize, gSize, bSize;

    switch (bpp)
    {
    case 8:
        rSize = 3;
        gSize = 3;
        bSize = 2;
    case 15:
    case 16:
        rSize = 5;
        gSize = 5;
        bSize = 5;
        break;
    default:
        rSize = 8;
        gSize = 8;
        bSize = 8;
        break;
    }
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, rSize);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, gSize);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, bSize);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 0);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    if (fsaa)
    {
        SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
        SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, fsaa);
    }

    videoFlags |= SDL_OPENGL;
    if (fullscreen)
        videoFlags |= SDL_FULLSCREEN;

    return SDL_SetVideoMode(width, height, bpp, videoFlags);
}


typedef struct {
    int bpp;
    int fullscreen;
} VIDEO_SETTINGS;

static VIDEO_SETTINGS sVideoSettings[] =
{
    { 32, 0 },
    { 24, 0 },
    { 16, 0 },
    { 32, 1 },
    { 24, 1 },
    { 16, 1 },
    { 32, 0 },
};
#define SVIDEOSETTINGS_COUNT (sizeof(sVideoSettings) / sizeof(sVideoSettings[0]))


int main(int argc, char *argv[])
{
    int a;

    if (SDL_Init(/*SDL_INIT_AUDIO | */SDL_INIT_VIDEO |
                 SDL_INIT_TIMER | SDL_INIT_NOPARACHUTE) < 0)
    {
        errorMessage("Unable to init SDL", SDL_GetError());
        exit(EXIT_FAILURE);
    }
    atexit(SDL_Quit);

    SDL_RWops *rw = SDL_RWFromFile(DATA_PREFIX "icon.png", "rb");
    SDL_Surface *icon = IMG_Load_RW(rw, 1);
    SDL_WM_SetIcon(icon, NULL);
    SDL_FreeSurface(icon);

    sScreen = NULL;
    for (a = 0; a < SVIDEOSETTINGS_COUNT; ++a)
    {
        VIDEO_SETTINGS *vs = &sVideoSettings[a];
        INT fs = vs->fullscreen;
        if (argc > 1 && argv[1][0] == 'w')
            fs = 0;
        else if (argc > 1 && argv[1][0] == 'f')
            fs = 1;
        sScreen = setVideoMode(WIDTH, HEIGHT, vs->bpp, fs, 0);
        const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();
        if (sScreen != NULL && (sScreen->flags & SDL_OPENGL) &&
            videoInfo/* && videoInfo->hw_available*/)
            break;
    }
    if (sScreen == NULL)
    {
        errorMessage("Unable to set video mode", SDL_GetError());
        exit(EXIT_FAILURE);
    }

    initSound();

    GameApp *app = new GameApp;
    BOOL result = app->init();

    if (result)
        app->run();

    deinitSound();

    return EXIT_SUCCESS;
}
